
// Game Settings globals
var gameMode = "PVP";  // PVP or AI
var aiDifficulty = "EASY"; // EASY or HARD
var playerNames = [];

var bots = [];
var nMoves = 5;
var nPlayers = 4;
var nBots = Math.floor(nPlayers / 2);
var currentPlayer = 0;
var epsilon = 1e-12;

var mousePosition = document.getElementById('mousePosition');
var message = document.getElementById('message');
var resetButton = document.getElementById('reset');
var scoreBox = document.getElementById('scores');
var canvas = document.getElementById('canvas');
var canvasWidth = canvas.width;
var canvasHeight = canvas.height;
var boardSize = canvasWidth;
var ctx = canvas.getContext('2d');

var moves = []; while(moves.push([]) < nPlayers);
var cache = [];
var scores = [];
var boardControl = [];
var light_colors = genColors(nPlayers, 50, 100);
var dark_colors = genColors(nPlayers, 100, 75);

function toggleAI() {
    var aiSettings = $("#aiSettings");
    var pvpSettings = $("#pvpSettings");
    var gameModeAI = $("#gameModeAI");
    var gameModePVP = $("#gameModePVP");

    if (!aiSettings.is(":visible")) {
        aiSettings.show();
        pvpSettings.hide();
        gameModeAI.addClass('active');
        gameModePVP.removeClass('active');
    } else {
        aiSettings.hide();
        pvpSettings.show();
        gameModeAI.removeClass('active');
        gameModePVP.addClass('active');
    }
    gameMode = "AI";
}

function togglePVP() {
    var pvpSettings = $("#pvpSettings");
    var aiSettings = $("#aiSettings");
    var gameModeAI = $("#gameModeAI");
    var gameModePVP = $("#gameModePVP");

    if (!pvpSettings.is(":visible")) {
        pvpSettings.show();
        aiSettings.hide();
        gameModeAI.removeClass('active');
        gameModePVP.addClass('active');
    } else {
        pvpSettings.hide();
        aiSettings.show();
        gameModeAI.addClass('active');
        gameModePVP.removeClass('active');
    }
    gameMode = "PVP";
}

function toggleAIEasy() {
    $("#aiModeEasy").addClass('active');
    $("#aiModeHard").removeClass('active');
    aiDifficulty = "EASY";
}

function toggleAIHard() {
    $("#aiModeHard").addClass('active');
    $("#aiModeEasy").removeClass('active');
    aiDifficulty = "HARD";
}

function saveSettings() {
    bots = [];
    // Initialize bots
    if (gameMode === "AI") {
        var pIndex;
        for (var i = 0; i < nBots; i++) {
            pIndex = i + 1;
            if (aiDifficulty === "EASY") {
                bots.push(new Bot("AIBotNovice" + pIndex, "EASY", pIndex));
            } else {
                bots.push(new Bot("AIBotSurvivor" + pIndex, "HARD", pIndex));
            }
        }
    }
    // Set new player names
    var botCount = 0;
    var name = "";
    for (var i = 0; i < nPlayers; i++) {
        if (i > 0 && botCount < bots.length) {
            name = bots[botCount++].botName;
        } else {
            name = $("#p" + (i + 1) + "Name").val();
        }
        if (name === "") {
            playerNames[i] = "Player " + (i + 1);
        } else {
            playerNames[i] = name;
        }
    }
    // Set number of stones
    nMoves = $("#stones").val();
    resetBoard();
}

function initSettings() {
    var aiSettings = $("#aiSettings");
    var pvpSettings = $("#pvpSettings");
    for (var i = 1; i <= nPlayers; i++) {
        playerNames.push("Player " + i);
    }
    // Game mode
    if (gameMode === "PVP") {
        togglePVP();
    } else {
        toggleAI();
    }
    $("#gameModeAI").on('click', function() { toggleAI(); });
    $("#gameModePVP").on('click', function() { togglePVP(); });

    // AI difficulty
    if (aiDifficulty === "EASY") {
        toggleAIEasy();
    } else {
        toggleAIHard();
    }
    $("#aiModeEasy").on('click', function() { toggleAIEasy(); });
    $("#aiModeHard").on('click', function() { toggleAIHard(); });
    // Save settings and restart
    $("#saveSettings").on('click', function() { saveSettings(); });
    // Number of players change
    $("#numplayers").change(function() {
        nPlayers = $("#numplayers").val();
        if (nPlayers == 2) {
            $("#player3").hide();
            $("#player4").hide();
        } else if (nPlayers == 3) {
            $("#player3").show();
            $("#player4").hide();
        } else if (nPlayers == 4) {
            $("#player3").show();
            $("#player4").show();
        }
        // number of bots is half the num players
        nBots = Math.floor(nPlayers / 2);
    });
}


function Bot(botName, difficulty, pIndex) {
    this.botName = botName;
    this.board = null;
    this.moves = [];
    this.difficulty = difficulty;
    this.pIndex = pIndex;
    this.curMove = 0;
}

function getBot(name) {
    for (var i = 0; i < bots.length; i++) {
        if (bots[i].botName === name) {
            return bots[i];
        }
    }
    return null;
}

Bot.prototype.randomMove = function() {
    var move_x, move_y;
    do {
        move_x = Math.floor(Math.random() * canvasWidth);
        move_y = Math.floor(Math.random() * canvasHeight);
    } while (!moveValid(move_x, move_y));
    return {
        x: move_x,
        y: move_y
    };
}

Bot.prototype.easyAI = function() {
    return this.randomMove();
}

// Returns the number of points in control by a player
Bot.prototype.controlArea = function(moves, pIndex) {
    var area = []; while(area.push(0) < nPlayers);
    for (var i = 0; i < boardSize; i++) {
        for (var j = 0; j < boardSize; j++) {
            var pulls = [];
            for (var p = 0; p < nPlayers; p++) {
                pulls[p] = 0;
            }
            for (var p = 0; p < nPlayers; p++) {
                for (var m=0; m<moves[p].length; m++) {
                    pulls[p] += cache[Math.abs(moves[p][m][0]-i)][Math.abs(moves[p][m][1]-j)];
                }
            }
            var max_val = pulls[0];
            var controllingPlayer = 0;
            for (var p = 1; p < nPlayers; p++) {
                if (pulls[p] > max_val) {
                    max_val = pulls[p];
                    controllingPlayer = p;
                }
            }
            area[controllingPlayer]++;
        }
    }
    return area[pIndex];
}

// Get a point nearest to the center of the canvas
Bot.prototype.getNearby = function(xcoord, ycoord) {
    var dir = 0, level = 0, x = xcoord, y = ycoord;
    var valid = moveValid(x, y);
    while (!valid) {
        if (dir === 0) {
            level++;
        }
        x = xcoord;
        y = ycoord;
        switch(dir){
            case 0:
                x += level;
                break;
            case 1:
                y += level;
                break;
            case 2:
                x -= level;
                break;
            case 3:
                y -= level;
                break;
        }
        valid = moveValid(x, y);
        dir = (dir + 1) % 4;
    }
    return {x: x, y: y};
}

// Get a copy of moves with move added to it
Bot.prototype.addMove = function(origMoves, move, pIndex) {
    var newMoves = []; while(newMoves.push([]) < nPlayers);
    // make a copy of moves
    for (var i = 0; i < nPlayers; i++) {
        for (var j = 0; j < origMoves[i].length; j++) {
            newMoves[i].push(origMoves[i][j]);
        }
    }
    newMoves[pIndex].push([move.x, move.y]);
    return newMoves;
}

// Nearby moving strategy where we find a location nearby
// our opponents and pick one that maximizes our score
Bot.prototype.nearbyMove = function() {
    var maxArea = this.controlArea(this.moves, this.pIndex);
    var pMoves = this.playerMoves(0);
    var nearby, newMoves, area, bestMove = null;
    // var pick1 = Math.floor(Math.random() * pMoves.length);
    for (var i = 0; i < pMoves.length; i++) {
        nearby = this.getNearby(pMoves[i].x, pMoves[i].y)
        newMoves = this.addMove(this.moves, nearby, this.pIndex);
        area = this.controlArea(newMoves, this.pIndex);
        if (area > maxArea) {
            bestMove = nearby;
            maxArea = area;
        }
    }
    return bestMove;
}

// Get locations of an player's stones
Bot.prototype.playerMoves = function(pIndex) {
    var moves = [];
    for (var m = 0; m < this.moves[pIndex].length; m++) {
        moves.push({
            x: this.moves[pIndex][m][0],
            y: this.moves[pIndex][m][1],
        });
    }
    return moves;
}

Bot.prototype.hardAI = function() {
    // First move always take as close to the center as possible
    if (this.curMove === 0)    {
        return this.getNearby(Math.floor(canvasWidth / 2 - 1),
            Math.floor(canvasHeight / 2 - 1));
    }
    // 1/10 chance of making a random move
    if (Math.random() < 0.10) {
        return this.randomMove();
    } else {
        return this.nearbyMove();
    }
}

Bot.prototype.nextMove = function(board, moves) {
    var move;
    this.board = board;
    this.moves = moves;
    if (this.difficulty === "EASY") {
        move = this.easyAI();
    } else {
        move = this.hardAI();
    }
    this.curMove++;
    return move;
}

function toHex(d) {
    return  ("0"+(Number(d).toString(16))).slice(-2).toUpperCase()
}

function hsvToRgb(h, s, v) {
    var r, g, b;
    var i;
    var f, p, q, t;

    // Make sure our arguments stay in-range
    h = Math.max(0, Math.min(360, h));
    s = Math.max(0, Math.min(100, s));
    v = Math.max(0, Math.min(100, v));

    // We accept saturation and value arguments from 0 to 100 because that's
    // how Photoshop represents those values. Internally, however, the
    // saturation and value are calculated from a range of 0 to 1. We make
    // That conversion here.
    s /= 100;
    v /= 100;

    if(s === 0) {
        // Achromatic (grey)
        r = g = b = v;
        return [Math.round(r * 255), Math.round(g * 255), Math.round(b * 255)];
    }

    h /= 60; // sector 0 to 5
    i = Math.floor(h);
    f = h - i; // factorial part of h
    p = v * (1 - s);
    q = v * (1 - s * f);
    t = v * (1 - s * (1 - f));

    switch(i) {
        case 0:
            r = v;
            g = t;
            b = p;
            break;

        case 1:
            r = q;
            g = v;
            b = p;
            break;

        case 2:
            r = p;
            g = v;
            b = t;
            break;

        case 3:
            r = p;
            g = q;
            b = v;
            break;

        case 4:
            r = t;
            g = p;
            b = v;
            break;

        default: // case 5:
            r = v;
            g = p;
            b = q;
    }

    return [Math.round(r * 255), Math.round(g * 255), Math.round(b * 255)];
}

function genColors(total, s, v) {
    var i = 360 / total; // distribute the colors evenly on the hue range
    var r = []; // hold the generated colors
    for (var x=0; x<total; x++)
    {
        r.push(hsvToRgb(i * x, s, v)); // you can also alternate the saturation and value for even more contrast between the colors
    }
    return r;
}

function color2String(color) {
    return "#" + toHex(color[0]) + toHex(color[1]) + toHex(color[2]);
}

// Maintain a cache of squared euclidean distances
function initCache() {
    // Instantiate 2D array
    while(cache.push([]) < boardSize);
    // Fill 2D array
    for (var i=0; i<boardSize; i++) {
        for (var j=0; j<boardSize; j++) {
            var d = i*i + j*j;
            if (i === 0 && j === 0) {
                cache[i].push(1000.0);
            } else {
                cache[i].push(1.0/d);
            }
        }
    }
}

// get mouse position
function getMousePosition(canvas, evt) {
    var x = new Number();
    var y = new Number();
    if (evt.pageX != undefined && evt.pageY != undefined) {
        x = evt.pageX;
        y = evt.pageY;
    } else {
        x = evt.clientX + document.body.scrollLeft
                + document.documentElement.scrollLeft;
        y = evt.clientY + document.body.scrollTop
                + document.documentElement.scrollTop;
    }
    x -= canvas.offsetLeft + 1;
    y -= canvas.offsetTop + 1;
    return {
        x: x,
        y: y
    };
}

function checkGameOver() {
    for (var i=0; i<nPlayers; i++) {
        if (moves[i].length < nMoves) {
            return false;
        }
    }
    return true;
}

function moveValid(x, y) {
    if (x >= boardSize || y >= boardSize || x < 0 || y < 0) {
        return false;
    }
    for (var i=0; i<nPlayers; i++) {
        for (var j=0; j<moves[i].length; j++) {
            if (moves[i][j][0] === x && moves[i][j][1] === y) {
                return false;
            }
        }
    }
    return true;
}

function clearBoard() {
    ctx.fillStyle="#ffffff";
    ctx.fillRect(0,0,canvasWidth,canvasHeight);
    ctx.fillStyle = "black";
    ctx.font = "bold 30px Arial";
    ctx.fillText("Click here to start", 110, 250);
}

function initBoard() {
    clearBoard();
    initCache();
}

function resetBoard() {
    clearBoard();
    moves = []; while(moves.push([]) < nPlayers);
    currentPlayer = 0;
    // message.innerHTML = "Player 1 to move";
    message.innerHTML = playerNames[currentPlayer] + " to move";
    scoreBox.innerHTML = "";
}

function drawStones() {
    for (var i=0; i<nPlayers; i++) {
        for (var j=0; j<moves[i].length; j++) {
            ctx.fillStyle = color2String(dark_colors[i]);
            ctx.beginPath();
            ctx.arc(moves[i][j][0], moves[i][j][1], 2, 0, Math.PI*2, true);
            ctx.closePath();
            ctx.fill();
        }
    }
}

function calculateControl() {
    boardControl = new Array();
    for (var i=0; i<boardSize; i++) {
        boardControl[i] = new Array();
        for (var j=0; j<boardSize; j++) {
            boardControl[i][j] = new Array();
        }
    }

    for (var i=0; i<boardSize; i++) {
        for (var j=0; j<boardSize; j++) {

            var pulls = [];
            for (var p=0; p<nPlayers; p++) {
                pulls[p] = 0;
            }

            for (var p=0; p<nPlayers; p++) {
                for (var m=0; m<moves[p].length; m++) {
                    pulls[p] += cache[Math.abs(moves[p][m][0]-i)][Math.abs(moves[p][m][1]-j)];
                }
            }

            var max_val = pulls[0];
            for (var p=1; p<nPlayers; p++) {
                if (pulls[p] > max_val) {
                    max_val = pulls[p];
                }
            }

            for (var p=0; p<nPlayers; p++) {
                if (Math.abs(max_val - pulls[p]) < epsilon) {
                    boardControl[i][j].push(p);
                }
            }

        }
    }
}

function drawBoard() {
    var imageData = ctx.getImageData(0, 0, canvasWidth, canvasHeight);

    var data = imageData.data;

    for (var y=0; y<canvasHeight; y++) {
        for (var x=0; x<canvasWidth; x++) {
            var index = (y * canvasWidth + x) * 4;

            var red = 0;
            var green = 0;
            var blue = 0;
            for (var i=0; i<boardControl[x][y].length; i++) {
                red += light_colors[boardControl[x][y][i]][0];
                green += light_colors[boardControl[x][y][i]][1];
                blue += light_colors[boardControl[x][y][i]][2];
            }
            var n = boardControl[x][y].length;
            data[index]   = red/n;
            data[++index] = green/n;
            data[++index] = blue/n;
            data[++index] = 225;
        }
    }

    ctx.putImageData(imageData, 0, 0);
}

function calculateScores() {
    scores = [];
    while(scores.push(0) < nPlayers);
    for (var x=0; x<canvasWidth; x++) {
        for (var y=0; y<canvasHeight; y++) {
            n = boardControl[x][y].length;
            for (var p=0; p<n; p++) {
                scores[boardControl[x][y][p]] += 1.0/n;
            }
        }
    }
}

function displayScores() {
    scoreHTML = "";
    for (var p=0; p<nPlayers; p++) {
        if (p === (currentPlayer+1)%nPlayers) {
            scoreHTML += "<div style=\"background-color:" +
                color2String(dark_colors[p]) + ";color:white;font-size:125%\">" +
                playerNames[p] + ": " + scores[p] + "</div>";
        } else {
            scoreHTML += "<div style=\"background-color:" +
                color2String(dark_colors[p]) + ";color:white\">" +
                playerNames[p] + ": " + scores[p] + "</div>";
        }
    }
    scoreBox.innerHTML = scoreHTML;
}

function saveScores() {
    if (checkGameOver()) {
        var winnerName = getWinner();
        var winnerScore = scores[playerNames.indexOf(winnerName)];
        //document.location.href="/drecco/index.php?task=GravitationalVoronoi&winner="+winnerName+"&ws="+winnerScore;
    }
}

function getWinner() {
    var winner, maxScore = 0;
    for (var i = 0; i < nPlayers; i++) {
        if (scores[i] > maxScore) {
            maxScore = scores[i];
            winner = i
        }
    }
    return playerNames[winner];
}

function updateGame(x, y) {
    moves[currentPlayer].push([x, y]);
    clearBoard();
    calculateControl();
    drawBoard();
    drawStones();
    calculateScores();
    displayScores();
    currentPlayer = (currentPlayer + 1) % nPlayers;
    message.innerHTML = playerNames[currentPlayer] + " to move";
}

canvas.addEventListener('mousemove', function(evt) {
    var mousePos = getMousePosition(canvas, evt);
    if (mousePos.x > -1 && mousePos.y > -1) {
        mousePosition.innerHTML = "x:" + mousePos.x + " y:" + mousePos.y;
    }
}, false);

canvas.addEventListener('mouseout', function(evt) {
    mousePosition.innerHTML = "x:? y:?";
}, false);

canvas.addEventListener('click', function(evt) {
    if (checkGameOver()) {
        message.innerHTML = getWinner() + " wins!"
        return;
    }
    var mousePos = getMousePosition(canvas, evt);
    if (!moveValid(mousePos.x, mousePos.y)) {
        message.innerHTML = "<div style=\"color:red;\">Invalid move</div>";
        return;
    }
    if (playerNames[currentPlayer].indexOf("AIBot") > -1) {
        message.innerHTML = "<div style=\"color:red;\">Not your turn</div>";
        return;
    }
    updateGame(mousePos.x, mousePos.y)
    // Check if its the bots turn
    do {
        var bot = getBot(playerNames[currentPlayer])
        if (bot !== null) {
            var move = bot.nextMove(boardControl, moves);
            updateGame(move.x, move.y);
        }
    } while(bot !== null);

    if (checkGameOver()) {
        message.innerHTML = getWinner() + " wins!"
    }
}, false);

resetButton.addEventListener('click', resetBoard, false);

saveButton.addEventListener('click', saveScores, false);

initSettings();
initBoard();
