import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import java.util.HashMap;
import java.util.ArrayList;
public class ScoreGen{
	private CityInfo cityData;

	public ScoreGen(String cityFileLoc) {
		//Loads cityfile into a HashMap
		//HashMap keys are a string of the name (in our case numbers)
		//HashMap value is an array of ints for x,y,z
		cityData = new CityInfo();

		try {
			FileReader fr = new FileReader(cityFileLoc);
			BufferedReader inData = new BufferedReader(fr);
			String tempLine;
			while ((tempLine = inData.readLine()) !=null) {
				tempLine.replace("\n", "");
				String[] tempData = tempLine.split(" ");
				cityData.addCity(tempData[0],
							Integer.parseInt(tempData[1]),
							Integer.parseInt(tempData[2]),
							Integer.parseInt(tempData[3]));
			}
			fr.close();
		}
		catch (IOException e) {
			e.printStackTrace();
		}
	}

	public double scorePathFile(String pathFileLoc) {
		//Load city path into ArrayList
		ArrayList<String> cityPath = new ArrayList<String>();
		try {
			FileReader fr = new FileReader(pathFileLoc);
			BufferedReader inData = new BufferedReader(fr);
			String tempLine;
			while ((tempLine = inData.readLine()) !=null) {
				tempLine.replace("\n", "");
				cityPath.add(tempLine);
			}
			fr.close();
		}
		catch (IOException e) {
			e.printStackTrace();
		}

		//Iterate through ArrayList scoring each
		String fromCity = cityPath.get(0);
		String toCity;
		double pathScore = 0.0;
		for (int i=1; i<cityPath.size(); i++) {
			toCity = cityPath.get(i);
			pathScore += cityData.cityToCity(fromCity, toCity);
			fromCity = toCity;
		}
		return pathScore;
	}

	public boolean wereAllVisited() {
		return cityData.allVisited();
	}

	public void clearVisits() {
		cityData.clearAllVisits();
	}
}

class CityInfo{
	private HashMap<String, int[]> cityData;
	//this is set to false if the path has a fake city
	private boolean fakeCityFlag;

	public CityInfo() {
		cityData = new HashMap<String, int[]>();
		fakeCityFlag = true;
	}

	public void addCity(String name, int x, int y, int z) {
		int[] values = {x,y,z,0};
		cityData.put(name, values);
	}

	public double cityToCity(String fromCity, String toCity) {
		double dist;
		int[] fromValues = cityData.get(fromCity);
		int[] toValues = cityData.get(toCity);

		if (fromValues!=null&&toValues!=null) {
			fromValues[3] = 1;
			toValues[3] = 1;

			 dist = Math.sqrt(
				Math.pow(toValues[0]-fromValues[0],2) + 
				Math.pow(toValues[1]-fromValues[1],2) + 
				Math.pow(toValues[2]-fromValues[2],2));
		}
		else {
			dist = 0;
			fakeCityFlag = false;
		}
		return dist;
	}

	public boolean allVisited() {
		boolean output = true;
		for (int[] vals : cityData.values()) {
			output = output&(vals[3] == 1);
		}
		output = output&fakeCityFlag;
		return output;
	}

	public void clearAllVisits() {
		for (String cityName : cityData.keySet()) {
			int[] values = cityData.get(cityName);
			values[3] = 0;
			cityData.put(cityName, values);
			fakeCityFlag = true;
		}
	}
}