package snow;

import java.awt.event.*;
import java.beans.*;
import javax.swing.*;
import static snow.SWConstants.*;

public class SWConfigureCostsDialog
  extends JDialog implements ActionListener, PropertyChangeListener
{
  SWGUIModel guiModel;
  JOptionPane optionPane;
  JTextField[] costFields = new JTextField[COSTS];
  String saveButton = "Save";
  String defaultsButton = "Defaults";
  String cancelButton = "Cancel";
  
  SWConfigureCostsDialog(SWGUIModel _guiModel, JFrame parent)
  {
    super(parent, true);
    for (int i = 0; i < COSTS; ++i)
    {
      costFields[i] = new JTextField(5);
      costFields[i].addActionListener(this);
    }
    setTitle("Configure Walk Through Costs");
    guiModel = _guiModel;
    String msg1 = "Specify the cost to walk through a building between:";
    JLabel spacer = new JLabel();
    String cornerAdjString = "adjacent corner entrances";
    String cornerOppString = "opposite corner entrances";
    String sideAdjString = "adjacent side entrances";
    String sideOppString = "opposite side entrances";
    String msg2 = "These settings only apply when \"Walk through buildings\" is enabled.\n";
    Object[] array =
      {
        msg1,
        spacer,
        cornerAdjString,
        costFields[CORNER_ADJACENT],
        cornerOppString,
        costFields[CORNER_OPPOSITE],
        sideAdjString,
        costFields[SIDE_ADJACENT],
        sideOppString,
        costFields[SIDE_OPPOSITE],
        msg2,
      };
    Object[] options = { saveButton, defaultsButton, cancelButton };
    optionPane =
      new JOptionPane(
        array,
        JOptionPane.PLAIN_MESSAGE,
        JOptionPane.YES_NO_CANCEL_OPTION,
        null,
        options,
        saveButton);
    setContentPane(optionPane);
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    addWindowListener(
      new WindowAdapter()
      {
        public void windowClosing(WindowEvent we)
        {
          /*
           * Instead of directly closing the window,
           * we're going to change the JOptionPane's
           * value property.
           */
          optionPane.setValue(new Integer(JOptionPane.CLOSED_OPTION));
        }
      });
    optionPane.addPropertyChangeListener(this);
    addComponentListener(
      new ComponentAdapter()
      {
        public void componentShown(ComponentEvent ce)
        {
          costFields[0].requestFocusInWindow();
        }
      });
    pack();
  }

  public void setVisible(boolean b)
  {
    if (b)
    {
      for (int i = 0; i < COSTS; ++i)
      {
        costFields[i].setText(Double.toString(guiModel.walkThroughCost(i)));
      }
    }
    super.setVisible(b);
  }

  public void actionPerformed(ActionEvent e)
  {
    optionPane.setValue(saveButton);
  }

  public void propertyChange(PropertyChangeEvent e)
  {
    String prop = e.getPropertyName();

    if (isVisible() &&
         (e.getSource() == optionPane) &&
         (JOptionPane.VALUE_PROPERTY.equals(prop) ||
           JOptionPane.INPUT_VALUE_PROPERTY.equals(prop)))
    {
      Object value = optionPane.getValue();
      if (value == JOptionPane.UNINITIALIZED_VALUE)
      {
        //ignore reset
        return;
      }

      //Reset the JOptionPane's value.
      //If you don't do this, then if the user
      //presses the same button next time, no
      //property change event will be fired.
      optionPane.setValue(JOptionPane.UNINITIALIZED_VALUE);

      double[] costs = new double[COSTS];
      if (saveButton.equals(value))
      {
        for (int i = 0; i < COSTS; ++i)
        {
          double val = Double.parseDouble(costFields[i].getText());
          if (val < 0)
          {
            costFields[i].selectAll();
            JOptionPane.showMessageDialog(
              null,
              "Costs must be non-negative",
              "Error",
              JOptionPane.ERROR_MESSAGE);
            costFields[i].requestFocusInWindow();
            return;
          }
          costs[i] = val;
        }
        guiModel.setWalkThroughCosts(costs);
      }
      else if (defaultsButton.equals(value))
      {
        guiModel.restoreWalkThroughDefaults();
      }
      // else cancel or close window - do nothing
      
      setVisible(false);
    }
  }
}
