package snow;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;

import javax.swing.*;
import javax.swing.event.*;

public class SnowWalkerFrame extends JFrame implements ChangeListener
{
  SWGUIModel guiModel;
  SWControlPanel controlPanel;
  SWMainPanel mainPanel;
  JDialog configureCostsDialog;
  JDialog setPuzzleDialog;
  JDialog scriptDialog;
  JDialog quickHelpDialog;
  Action configureCostsAction;
  Action setPuzzleDialogAction;
  Action scriptDialogAction;
  Action quickHelpAction;
  AppletContext appletContext;
  JPanel panel;

  SnowWalkerFrame(AppletContext _appletContext)
  {
    super("Snow Walkers");
    appletContext = _appletContext;
    guiModel = new SWGUIModel();
    guiModel.addChangeListener(this);
    makeDialogs();
    mainPanel = new SWMainPanel(guiModel);
    controlPanel = new SWControlPanel(this);
    createMenu();
    panel = new JPanel(new BorderLayout());
    panel.setBorder(BorderFactory.createEmptyBorder());
    panel.add(mainPanel, BorderLayout.CENTER);
    panel.add(controlPanel, BorderLayout.WEST);
    JScrollPane scroll = new JScrollPane(panel);
    scroll.setViewportBorder(BorderFactory.createEmptyBorder());
    add(scroll);
    pack();
    setVisible(true);
  }
  
  void makeDialogs()
  {
    configureCostsDialog = new SWConfigureCostsDialog(guiModel, this);
    setPuzzleDialog = new SWSetPuzzleDialog(guiModel, this);
    scriptDialog = new SWScriptDialog(guiModel, this);
    quickHelpDialog = new SWQuickHelpDialog(this);
    configureCostsAction =
      new ShowDialogAction("Configure Costs...", configureCostsDialog);
    setPuzzleDialogAction =
      new ShowDialogAction("Set Puzzle...", setPuzzleDialog);
    scriptDialogAction =
      new ShowDialogAction("Run Script...", scriptDialog);
    quickHelpAction =
      new ShowDialogAction("Quick Help", quickHelpDialog);
  }
  
  void createMenu()
  {
    JMenuBar menubar = new JMenuBar();
    JMenu actionsMenu = new JMenu("Actions");
    actionsMenu.setMnemonic(KeyEvent.VK_A);
    JMenuItem setPuzzleItem = new JMenuItem(setPuzzleDialogAction);
    JMenuItem configureCostsItem = new JMenuItem(configureCostsAction);
    JMenuItem runScriptItem = new JMenuItem(scriptDialogAction);
    JMenuItem undoItem = new JMenuItem(guiModel.undoAction);
    JMenuItem resetItem = new JMenuItem(guiModel.resetAction);
    actionsMenu.add(setPuzzleItem);
    actionsMenu.add(configureCostsItem);
    actionsMenu.add(runScriptItem);
    actionsMenu.add(undoItem);
    actionsMenu.add(resetItem);
    if (appletContext != null)
    {
      actionsMenu.add(new CloseAction());
    }
    else
    {
      actionsMenu.add(new ExitAction());
    }
    menubar.add(actionsMenu);
    JMenu helpMenu = new JMenu("Help");
    helpMenu.setMnemonic(KeyEvent.VK_H);
    JMenuItem quickHelp = new JMenuItem(quickHelpAction);
    helpMenu.add(quickHelp);
    if (appletContext != null)
    {
      URL puzzleURL = null;
      try
      {
        puzzleURL =
          new URL("http://www.cims.nyu.edu/~ah203/sw/description.html");
        Action showPuzzleAction =
          new ShowURLAction(
            "Open Puzzle Description in Browser...",
            puzzleURL);
        helpMenu.add(showPuzzleAction);
      }
      catch (MalformedURLException mux)
      {
        // shouldn't happen, but do nothing
      }
    }
    menubar.add(helpMenu);
    setJMenuBar(menubar);
  }
  
  public static void main(String args[])
  {
    new SnowWalkerFrame(null);
  }
  
  class ShowDialogAction extends AbstractAction
  {
    JDialog dialog;
    
    ShowDialogAction(String text, JDialog _dialog)
    {
      super(text);
      dialog = _dialog;
    }
    
    public void actionPerformed(ActionEvent e)
    {
      dialog.setVisible(true);
    }
  }
  
  class ShowURLAction extends AbstractAction
  {
    URL url;
    
    ShowURLAction(String text, URL _url)
    {
      super(text);
      url = _url;
    }

    public void actionPerformed(ActionEvent e)
    {
      appletContext.showDocument(url, "_blank");
    }
  }

  class ExitAction extends AbstractAction
  {
    ExitAction()
    {
      super("Exit");
    }

    public void actionPerformed(ActionEvent e)
    {
      System.exit(0);
    }
  }
  
  class CloseAction extends AbstractAction
  {
    CloseAction()
    {
      super("Close");
    }
    
    public void actionPerformed(ActionEvent e)
    {
      dispose();
    }
  }

  public void stateChanged(ChangeEvent e)
  {
    panel.revalidate();
  }
}
